#include <stdio.h>
#include <math.h>
#include <stdlib.h>

typedef struct {
	float x;
	float y;
} TPoint;

typedef enum {Less, Equal, More, NotEqual, NotComp} CmpType;

typedef struct PLACE_HOLDER *PObject;

typedef CmpType (*PCmpFunction) (PObject U, PObject V);

typedef void (*PProcedure) (PObject);

typedef void (*PDrawProc) ();

typedef struct PLACE_HOLDER {
	PProcedure Draw;
	PCmpFunction Compare;
} TObject;

typedef struct {
	PProcedure Draw;
	PCmpFunction Compare;
	TPoint Center;
	float Radius;
} TCircle;
typedef TCircle *PCircle;

typedef struct {
	PProcedure Draw;
	PCmpFunction Compare;
	TPoint A, B, C;
} TTriangle;
typedef TTriangle *PTriangle;

typedef struct {
	PProcedure Draw;
	PCmpFunction Compare;
	PDrawProc ProcPtr;
} TExecutable;
typedef TExecutable *PExecutable;

typedef PObject TValue;

void DrawCircle (PObject P)
{
	PCircle T;
	T = (PCircle) P;
	printf("ОКРУЖНОСТЬ(X=%f, Y=%f, R=%f)", T->Center.x, T->Center.y, T->Radius);
}

void DrawTriangle (PObject P)
{
	PTriangle T;
	T = (PTriangle) P;
	printf("ТРЕУГ(A=(%f, %f),", T->A.x, T->A.y);
	printf("ТРЕУГ(A=(%f, %f),", T->B.x, T->B.y);
	printf("ТРЕУГ(A=(%f, %f)\n", T->C.x, T->C.y);
}

void DrawExecutable (PObject P)
{
	if (((PExecutable) P)->ProcPtr != NULL)
		(((PExecutable) P)->ProcPtr) ();
}

int EqType (PObject U, PObject V)
{
	return U->Draw == V->Draw;
}

CmpType CmpCircle (PObject U, PObject V)
{
	if (!EqType(U, V))
		return NotComp;
	else
	{
		if (((PCircle) U)->Radius == ((PCircle) V)->Radius)
			return Equal;
		if (((PCircle) U)->Radius < ((PCircle) V)->Radius)
			return Less;
		if (((PCircle) U)->Radius > ((PCircle) V)->Radius)
			return More;
	}
}

float Dist (TPoint A, TPoint B)
{
	return sqrt(pow(A.x-B.x, 2)+pow(A.y-B.y, 2));
}

float Sum (TTriangle U)
{
	return Dist(U.A, U.B) + Dist(U.A, U.C) + Dist(U.B, U.C);
}

CmpType CmpTriangle (PObject U, PObject V)
{
	if (!EqType(U, V))
		return NotComp;
	else
	{
		if (Sum(*((PTriangle) U)) == Sum(*((PTriangle) V)))
			return Equal;
		if (Sum(*((PTriangle) U)) < Sum(*((PTriangle) V)))
			return Less;
		if (Sum(*((PTriangle) U)) > Sum(*((PTriangle) V)))
			return More;
	}
}

CmpType CmpExecutable (PObject U, PObject V)
{
	if (!EqType(U, V))
		return NotComp;
	else if (((PExecutable) U)->ProcPtr == ((PExecutable) V)->ProcPtr)
		return Equal;
	else
		return NotEqual;
}

TValue NewCircle (float x, float y, float r)
{
	PCircle Z;
	Z = malloc(sizeof(TCircle));
	Z->Center.x = x;
	Z->Center.y = y;
	Z->Radius = r;
	Z->Draw = DrawCircle;
	Z->Compare = CmpCircle;
	return (TValue) Z;
}

TValue NewTriangle (float ax, float ay, float bx, float by, float cx, float cy)
{
	PTriangle Z;
	Z = malloc(sizeof(TTriangle));
	Z->A.x = ax;
	Z->A.y = ay;
	Z->B.x = bx;
	Z->B.y = by;
	Z->C.x = cx;
	Z->C.y = cy;
	Z->Draw = DrawTriangle;
	Z->Compare = CmpTriangle;
	return (TValue) Z;
}

TValue NewExecutable (PProcedure T)
{
	PExecutable Z;
	Z->ProcPtr = T;
	Z->Draw = DrawExecutable;
	Z->Compare = CmpExecutable;
	return (TValue) Z;
}



int main ()
{
	return 0;
}
